#!/bin/bash

#
#  Copyright Red Hat, Inc. 2006
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the
#  Free Software Foundation; either version 2, or (at your option) any
#  later version.
#
#  This program is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; see the file COPYING.  If not, write to the
#  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
#  MA 02139, USA.
#
#
#  Author(s):
#	Marek Grac (mgrac at redhat.com)
#
#  Version: 0.5
#

export LC_ALL=C
export LANG=C
export PATH=/bin:/sbin:/usr/bin:/usr/sbin

declare APACHE_HTTPD=/usr/sbin/httpd
declare APACHE_genConfig="/tmp/httpd.$OCF_RESKEY_name"
declare APACHE_pidFile
declare APACHE_serverConfigFile
declare APACHE_defaultPidFile=run/httpd.pid

. $(dirname $0)/ocf-shellfuncs
. $(dirname $0)/util/config-utils.sh

declare APACHE_parseConfig=$(dirname $0)/util/httpd-parse-config.pl

apache_pidFile()
{
	declare tmpFile="/tmp/httpd.$OCF_RESKEY_name.$$"
	declare CFG_pidFile
	declare CFG_serverRoot

	if [ -f "$APACHE_genConfig" ]; then
		sha1_verify "$APACHE_genConfig"
		if [ $? -ne 0 ]; then
			ocf_log debug "Config file changed; skipping"
			cp "$APACHE_genConfig" "$tmpFile"
		else 
			generate_configFile "$APACHE_serverConfigFile" "$tmpFile"
		fi
	else
		generate_configFile "$APACHE_serverConfigFile" "$tmpFile"
	fi	

	CFG_pidFile=`grep -e '^PidFile' "$tmpFile" | head -n 1 | \
		sed 's/^PidFile "\(.*\)"/\1/;s/^PidFile \(.*\)/\1/'`
	CFG_serverRoot=`grep -e '^ServerRoot' "$tmpFile" | head -n 1 | \
		sed 's/^ServerRoot "\(.*\)"/\1/;s/^ServerRoot \(.*\)/\1/'`

	if [ -z "$CFG_pidFile" ]; then
		CFG_pidFile="$APACHE_defaultPidFile"
	fi

	if [ -z "$CFG_serverRoot" ]; then
		CFG_serverRoot="$OCF_RESKEY_serverRoot"
	fi

	if [[ "$CFG_pidFile" =~ '^/' ]]; then
		APACHE_pidFile="$CFG_pidFile"
	else 
		APACHE_pidFile="$CFG_serverRoot/$CFG_pidFile"
	fi

	rm -f "$tmpFile"

	return;
}

apache_serverConfigFile()
{
	if [[ "$OCF_RESKEY_serverConfigFile" =~ '^/' ]]; then
		APACHE_serverConfigFile="$OCF_RESKEY_serverConfigFile"
	else 
		APACHE_serverConfigFile="$OCF_RESKEY_serverRoot/$OCF_RESKEY_serverConfigFile"
	fi

	return;
}

verify_all()
{
	verify_serverRoot || return $OCF_ERR_ARGS
	verify_serverConfigFile || return $OCF_ERR_ARGS
	verify_pidFile || return $OCF_ERR_ARGS
	verify_syntaxConfig || return $OCF_ERR_ARGS

	return 0
}

verify_serverConfigFile() 
{
	ocf_log debug "Checking: ServerConfigFile"

	[ -n "$OCF_RESKEY_serverConfigFile" ] || return $OCF_ERR_ARGS

	[ -r "$APACHE_serverConfigFile" ] || return $OCF_ERR_ARGS

	ocf_log debug "Checking: ServerConfigFile > succeed"

	return 0
}

verify_syntaxConfig()
{
	ocf_log debug "Checking syntax of the configuration file"

	"$APACHE_HTTPD" -t \
		-D"$OCF_RESKEY_name" \
		-d "$OCF_RESKEY_serverRoot" \
		-f "$APACHE_serverConfigFile" &> /dev/null
		
	if [ $? -ne 0 ]; then
		return $OCF_ERR_GENERIC
	fi

	ocf_log debug "Checking syntax of the configuration file > succeed"

	return 0
}

verify_pidFile() 
{
	ocf_log debug "Checking: PID file"

	[ -n "$APACHE_pidFile" ] || return $OCF_ERR_ARGS

	ocf_log debug "Checking: PID file > succeed"

	return 0
}

verify_serverRoot()
{
	ocf_log debug "Checking: ServerRoot"
	[ -n "$OCF_RESKEY_serverRoot" ] || return $OCF_ERR_ARGS
	[ -d "$OCF_RESKEY_serverRoot" ] || return $OCF_ERR_ARGS

	ocf_log debug "Checking: ServerRoot > succeed"

	return 0
}

generate_configFile()
{
	declare originalConfigFile=$1;
	declare generatedConfigFile=$2;
	declare ip_addresses=$3;

	if [ -f "$generatedConfigFile" ]; then
		sha1_verify "$generatedConfigFile"
		if [ $? -ne 0 ]; then
			ocf_log debug "Config file changed; skipping"
			return 0
		fi
	fi	

	ocf_log debug "Generating new config file to $2 from $1"

	generate_configTemplate "$generatedConfigFile" "$1"
	cat >> "$generatedConfigFile" << EOT
# From a cluster perspective, the key fields are:
#     Listen - must be set to service floating IP address.
#     PidFile - path to the file where PID of running application is stored
#     ServerRoot - path to the ServerRoot (initial value is set in service conf)
#

EOT

	IFS_old="$IFS"
	IFS=$'\n'
	for i in `"$APACHE_parseConfig" -D"$OCF_RESKEY_name" < "$originalConfigFile" | grep -P '(^Listen)|(^Port)' | grep -v ':'`; do 
		port=`echo $i | sed 's/^Listen \(.*\)/\1/;s/^Port \(.*\)/\1/'`;
		IFS=$' ';
		for z in $ip_addresses; do 
			echo "Listen $z:$port" >> "$generatedConfigFile";
		done
		IFS=$'\n';
	done;
	IFS="$IFS_old"

	cat "$originalConfigFile" | sed 's/^Listen/### Listen/;s/^Port/### Port/' | \
	"$APACHE_parseConfig" -D"$OCF_RESKEY_name" >> "$generatedConfigFile"

	sha1_addToFile "$generatedConfigFile"
	ocf_log debug "Generating config file > succeed"
}

start()
{
	declare ccs_fd
	declare ip_addresses

	
	ocf_log info "Starting service \"$OCF_RESOURCE_INSTANCE\""

	ocf_log debug "Checking: existence of PID file"
	if [ -e "$APACHE_pidFile" ]; then
		ocf_log error "Apache PID file exists for \"$OCF_RESOURCE_INSTANCE\""
		return $OCF_GENERIC_ERROR
	fi
	ocf_log debug "Checking: existence of PID file > succeed"

	ccs_fd=$(ccs_connect);
	if [ $? -ne 0 ]; then
		ocf_log error "Unable to connect to ccs in \"$OCF_RESOURCE_INSTANCE\""
		return $OCF_GENERIC_ERROR
	fi
	get_service_ip_keys "$ccs_fd" "$OCF_RESKEY_service_name"
	ip_addresses=`build_ip_list "$ccs_fd"`

	generate_configFile "$APACHE_serverConfigFile" "$APACHE_genConfig" "$ip_addresses"

	"$APACHE_HTTPD" \
		"-D$OCF_RESKEY_name" \
		-d "$OCF_RESKEY_serverRoot" \
		-f "$APACHE_genConfig" \
		-k start

	if [ $? -ne 0 ]; then
		ocf_log err "Service \"$OCF_RESOURCE_INSTANCE\" failed: \
		$APACHE_HTTPD "-D$OCF_RESKEY_name" \
		-d "$OCF_RESKEY_serverRoot" \
		-f "$APACHE_genConfig" -k start"			
		return $OCF_ERR_GENERIC
	else
		ocf_log debug "Service \"$OCF_RESOURCE_INSTANCE\" started"
	fi

	return 0;
}

stop()
{
	ocf_log info "Stopping service \"$OCF_RESOURCE_INSTANCE\""

	ocf_log debug "Checking: existence of PID file"
	if [ ! -e "$APACHE_pidFile" ]; then
		ocf_log error "Apache PID file is missing for service \"$OCF_RESOURCE_INSTANCE\""
		return $OCF_GENERIC_ERROR
	fi
	ocf_log debug "Checking: existence of PID file > succeed"

	"$APACHE_HTTPD" -k stop

	if [ $? -ne 0 ]; then
		ocf_log err "Failed to stop the service \"$OCF_RESOURCE_INSTANCE\": \
		"$APACHE_HTTPD" -k stop"
		return $OCF_GENERIC_ERROR
	else
		ocf_log debug "Service \"$OCF_RESOURCE_INSTANCE\" stopped"
	fi
	
	return 0;
}

status()
{
	ocf_log info "Monitoring service \"$OCF_RESOURCE_INSTANCE\""
	
	ocf_log debug "Checking: existence of PID file"
	if [ ! -e "$APACHE_pidFile" ]; then
		ocf_log error "Apache PID file for service \"$OCF_RESOURCE_INSTANCE\" is missing"
		return $OCF_GENERIC_ERROR
	fi
	ocf_log debug "Checking: existence of PID file > succeed"

	ocf_log debug "Checking: status of service"
	if [ ! -d /proc/`cat "$APACHE_pidFile"` ]; then
		ocf_log error "Service \"$OCF_RESOURCE_INSTANCE\" is not running"
		return $OCF_GENERIC_ERROR
	fi	

	ocf_log debug "Service \"$OCF_RESOURCE_INSTANCE\" is running"

	return 0
}

if [ "$1" != "meta-data" ]; then
	apache_serverConfigFile
	apache_pidFile
fi;
		
case $1 in
	meta-data)
		cat $(dirname $0)/apache.metadata
		exit 0
		;;
	verify-all)
		verify_all
		exit $?
		;;
	start)
		verify_all && start
		exit $?
		;;
	stop)
		verify_all && stop
		exit $?
		;;
	status|monitor)
		verify_all
		status
		exit $?
		;;
	restart)
		verify_all
		stop
		start
		exit $?
		;;
	*)
		echo "usage: $0 {start|stop|status|monitor|restart|meta-data|verify-all}"
		exit $OCF_ERR_GENERIC
		;;
esac
