#!/bin/bash

#
#  Copyright Red Hat, Inc. 2006
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the
#  Free Software Foundation; either version 2, or (at your option) any
#  later version.
#
#  This program is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; see the file COPYING.  If not, write to the
#  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
#  MA 02139, USA.
#
#
#  Author(s):
#	Marek Grac (mgrac at redhat.com)
#
#  Version: 0.5
#

export LC_ALL=C
export LANG=C
export PATH=/bin:/sbin:/usr/bin:/usr/sbin

declare MYSQL_MYSQLD=/usr/bin/mysqld_safe
declare MYSQL_ipAddress
declare MYSQL_pidFile="/var/run/mysqld/mysql.$OCF_RESKEY_name.pid"

. $(dirname $0)/ocf-shellfuncs
. $(dirname $0)/util/config-utils.sh

verify_all()
{
	verify_name || return $OCF_ERR_ARGS
	verify_configFile || return $OCF_ERR_ARGS
	verify_pidFile || return $OCF_ERR_ARGS

	return 0
}

verify_configFile() 
{
	ocf_log debug "Checking: Configuration file"

	[ -n "$OCF_RESKEY_configFile" ] || return $OCF_ERR_ARGS

	[ -r "$OCF_RESKEY_configFile" ] || return $OCF_ERR_ARGS

	ocf_log debug "Checking: Configuration file > succeed"

	return 0
}

verify_pidFile() 
{
	ocf_log debug "Checking: PID file"

	[ -n "$MYSQL_pidFile" ] || return $OCF_ERR_ARGS

	ocf_log debug "Checking: PID file > succeed"

	return 0
}

verify_name() 
{
	ocf_log debug "Checking: Name of service"

	[ -n "$OCF_RESKEY_name" ] || return $OCF_ERR_ARGS

	ocf_log debug "Checking: Name of service > succeed"

	return 0
}

start()
{
	declare ccs_fd;
	
	ocf_log info "Starting service \"$OCF_RESOURCE_INSTANCE\""

	ocf_log debug "Checking: existence of PID file"
	if [ -e "$MYSQL_pidFile" ]; then
		ocf_log error "PID file exist for service \"$OCF_RESOURCE_INSTANCE\""
		return $OCF_GENERIC_ERROR
	fi
	ocf_log debug "Checking: existence of PID file > succeed"

	ocf_log debug "Checking: IP address"
	if [ -n "$OCF_RESKEY_ipAddress" ]; then
		MYSQL_ipAddress="$OCF_RESKEY_ipAddress"
	else
	        ccs_fd=$(ccs_connect);
	        if [ $? -ne 0 ]; then
        	        ocf_log error "Unable to connect to ccs in \"$OCF_RESOURCE_INSTANCE\""
	                return $OCF_GENERIC_ERROR
	        fi

	        get_service_ip_keys "$ccs_fd" "$OCF_RESKEY_service_name"
	        ip_addresses=`build_ip_list "$ccs_fd"`

		if [ -n "$ip_addresses" ]; then
			for i in $ip_addresses; do
				MYSQL_ipAddress="$i"
				break;
			done
		else
			ocf_log error "No IP address found for service \"$OCF_RESOURCE_INSTANCE\""
		fi
	fi

	ocf_log debug "Checking: IP address > succeed"

	# mysql start
	$MYSQL_MYSQLD --defaults-file="$OCF_RESKEY_configFile" \
		--pid-file="$MYSQL_pidFile" \
		--bind-address="$MYSQL_ipAddress" > /dev/null 2>&1 &

	if [ $? -ne 0 ]; then
		ocf_log err "Failed to start the service \"$OCF_RESOURCE_INSTANCE\""
		return $OCF_GENERIC_ERROR
	fi

	declare timeout=30
	while [ "$timeout" -gt 0 ]; do
		if [ -f "$MYSQL_pidFile" ]; then
			break;			
		fi
		sleep 1
		let timeout=${timeout}-1
        done

        if [ "$timeout" -eq 0 ]; then
                ocf_log err "Timeout error occurred trying to start MySQL Daemon for \"$OCF_RESOURCE_INSTANCE\""
		ocf_log err "Failed to start the service \"$OCF_RESOURCE_INSTANCE\""
		return $OCF_GENERIC_ERROR
	fi
	
	ocf_log debug "Service \"$OCF_RESOURCE_INSTANCE\" started"

	return 0;
}

stop()
{
	ocf_log info "Stopping service \"$OCF_RESOURCE_INSTANCE\""

	ocf_log debug "Checking: existence of PID file"
	if [ ! -e "$MYSQL_pidFile" ]; then
		ocf_log error "PID file is missing for service \"$OCF_RESOURCE_INSTANCE\""
		return $OCF_GENERIC_ERROR
	fi
	ocf_log debug "Checking: existence of PID file > succeed"

	kill `cat "$MYSQL_pidFile"`

	if [ $? -ne 0 ]; then
		ocf_log err "Failed to stop the service \"$OCF_RESOURCE_INSTANCE\""
		return $OCF_GENERIC_ERROR
	else
		ocf_log debug "Service \"$OCF_RESOURCE_INSTANCE\" stopped"
	fi
	
	return 0;
}

status()
{
	ocf_log info "Monitoring service \"$OCF_RESOURCE_INSTANCE\""
	
	ocf_log debug "Checking: existence of PID file"
	if [ ! -e "$MYSQL_pidFile" ]; then
		ocf_log error "PID file for service \"$OCF_RESOURCE_INSTANCE\" is missing"
		return $OCF_GENERIC_ERROR
	fi
	ocf_log debug "Checking: existence of PID file > succeed"

	ocf_log debug "Checking: status of service"
	if [ ! -d /proc/`cat "$MYSQL_pidFile"` ]; then
		ocf_log error "Service \"$OCF_RESOURCE_INSTANCE\" is not running"
		return $OCF_GENERIC_ERROR
	fi	

	ocf_log debug "Service \"$OCF_RESOURCE_INSTANCE\" is running"

	return 0
}

case $1 in
	meta-data)
		cat $(dirname $0)/mysql.metadata
		exit 0
		;;
	verify-all)
		verify_all
		exit $?
		;;
	start)
		verify_all && start
		exit $?
		;;
	stop)
		verify_all && stop
		exit $?
		;;
	status|monitor)
		verify_all
		status
		exit $?
		;;
	restart)
		verify_all
		stop
		start
		exit $?
		;;
	*)
		echo "usage: $0 {start|stop|status|monitor|restart|meta-data|verify-all}"
		exit $OCF_ERR_GENERIC
		;;
esac
